from TransformerAI.Classes import *
from tensorflow.keras.models import load_model
from tokenizers import Tokenizer
import os
import pickle
import random
import sys
import io
import json
import tensorflow as tf

# Настройка кодировки UTF-8
sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')

script_directory = os.path.dirname(os.path.abspath(__file__))

# Загружаем классы тегов
with open(os.path.join(script_directory, 'names_tags.pkl'), 'rb') as f:
    names_tags = pickle.load(f)

# Загружаем модель
savemodel = load_model(
    os.path.join(script_directory, "Transformer4.keras"),
    custom_objects={
        'Encoder': Encoder,
        'ModifiedModel': ModifiedModel,
        'EncoderLayer': EncoderLayer,
        'CustomMultiHeadAttention': CustomMultiHeadAttention,
    }
)

# Загружаем токенизатор
tokenizer = Tokenizer.from_file(os.path.join(script_directory, "tokenizer.json"))

# Загружаем сценарии
with open(os.path.join(script_directory, 'scenarios.json'), encoding='utf-8') as f:
    intents_json = json.load(f)

# Функция для предсказания класса предложения
def predict_sentence_class(model, sentence, tokenizer, label_names):
    max_len = model.encoder.pos_encoding.shape[1]

    # Токенизация
    sequence = tokenizer.encode(sentence).ids
    padded_seq = tf.keras.preprocessing.sequence.pad_sequences([sequence], maxlen=max_len, padding='post')

    # Создание маски
    mask = create_padding_mask(tf.constant(padded_seq))

    # Предсказание
    predictions = list(model.predict([padded_seq, mask])[0][0])
    max_value_predictions = max(predictions)
    max_index_predictions = predictions.index(max_value_predictions)
    return max_index_predictions

# Функция для получения ответа на основе предсказаний
def get_response(max_index_predictions, intents_json):
    response_of_our_tag = random.choice(intents_json['intents'][max_index_predictions]['responses'])
    return response_of_our_tag

# Функция для генерации ответа ИИ
def AI_response(textRequest):
    # Делаем предсказание и получаем индекс
    max_index_predictions = predict_sentence_class(savemodel, textRequest, tokenizer, names_tags)
    # Получаем ответ на основе индекса предсказания
    response = get_response(max_index_predictions, intents_json)
    return response

# Тестируем функцию
#pre = AI_response("Привет")
#print(pre)